@@ -35,6 +35,8 @@ const (
 
 	KluctlRequestReconcileAnnotation = "kluctl.io/request-reconcile"
 	KluctlRequestDeployAnnotation    = "kluctl.io/request-deploy"
+	KluctlRequestPruneAnnotation     = "kluctl.io/request-prune"
+	KluctlRequestValidateAnnotation  = "kluctl.io/request-validate"
 )
 
 type KluctlDeploymentSpec struct {
@@ -100,7 +102,7 @@ type KluctlDeploymentSpec struct {
 	// Specifies the kubeconfig to be used when invoking kluctl. Contexts in this kubeconfig must match
 	// the context found in the kluctl target. As an alternative, specify the context to be used via 'context'
 	// +optional
-	KubeConfig *KubeConfig `json:"kubeConfig"`
+	KubeConfig *KubeConfig `json:"kubeConfig,omitempty"`
 
 	// Target specifies the kluctl target to deploy. If not specified, an empty target is used that has no name and no
 	// context. Use 'TargetName' and 'Context' to specify the name and context in that case.
@@ -123,7 +125,7 @@ type KluctlDeploymentSpec struct {
 	// Args specifies dynamic target args.
 	// +optional
 	// +kubebuilder:pruning:PreserveUnknownFields
-	Args runtime.RawExtension `json:"args,omitempty"`
+	Args *runtime.RawExtension `json:"args,omitempty"`
 
 	// Images contains a list of fixed image overrides.
 	// Equivalent to using '--fixed-images-file' when calling kluctl.
@@ -208,6 +210,20 @@ type KluctlDeploymentSpec struct {
 	// +kubebuilder:default:=false
 	// +optional
 	Delete bool `json:"delete,omitempty"`
+
+	// Manual enables manual deployments, meaning that the deployment will initially start as a dry run deployment
+	// and only after manual approval cause a real deployment
+	// +optional
+	Manual bool `json:"manual,omitempty"`
+
+	// ManualObjectsHash specifies the rendered objects hash that is approved for manual deployment.
+	// If Manual is set to true, the controller will skip deployments when the current reconciliation loops calculated
+	// objects hash does not match this value.
+	// There are two ways to use this value properly.
+	// 1. Set it manually to the value found in status.lastObjectsHash.
+	// 2. Use the Kluctl Webui to manually approve a deployment, which will set this field appropriately.
+	// +optional
+	ManualObjectsHash *string `json:"manualObjectsHash,omitempty"`
 }
 
 // GetRetryInterval returns the retry interval
@@ -225,20 +241,46 @@ type ProjectSource struct {
 
 	// Ref specifies the branch, tag or commit that should be used. If omitted, the default branch of the repo is used.
 	// +optional
-	Ref *GitRef `json:"ref,omitempty"`
+	Ref *types.GitRef `json:"ref,omitempty"`
 
 	// Path specifies the sub-directory to be used as project directory
 	// +optional
 	Path string `json:"path,omitempty"`
 
+	// SecretRef specifies the Secret containing authentication credentials for
+	// See GitCredentials.SecretRef for details
+	// DEPRECATED this field is deprecated and will be removed in a future version of the controller. Use Credentials
+	// instead.
+	// WARNING using this field causes the controller to pass http basic auth credentials to ALL repositories involved.
+	// Use Credentials with a proper Host field instead.
+	SecretRef *LocalObjectReference `json:"secretRef,omitempty"`
+
+	// Credentials specifies a list of secrets with credentials
+	// +optional
+	Credentials []GitCredentials `json:"credentials,omitempty"`
+}
+
+type GitCredentials struct {
+	// Host specifies the hostname that this git secret applies to. If set to '*', this set of credentials
+	// applies to all hosts.
+	// Using '*' for http(s) based repositories is not supported, meaning that such credentials sets will be ignored.
+	// You must always set a proper hostname in that case.
+	// +required
+	Host string `json:"host,omitempty"`
+
+	// PathPrefix specified the path prefix to be used to filter git urls. Only urls that have this prefix will use
+	// this set of credentials.
+	// +optional
+	PathPrefix string `json:"pathPrefix,omitempty"`
+
 	// SecretRef specifies the Secret containing authentication credentials for
 	// the git repository.
 	// For HTTPS repositories the Secret must contain 'username' and 'password'
 	// fields.
 	// For SSH repositories the Secret must contain 'identity'
 	// and 'known_hosts' fields.
-	// +optional
-	SecretRef *LocalObjectReference `json:"secretRef,omitempty"`
+	// +required
+	SecretRef LocalObjectReference `json:"secretRef"`
 }
 
 // Decryption defines how decryption is handled for Kubernetes manifests.
@@ -298,7 +340,13 @@ type KluctlDeploymentStatus struct {
 	LastHandledReconcileAt string `json:"lastHandledReconcileAt,omitempty"`
 
 	// +optional
-	LastHandledDeployAt string `json:"LastHandledDeployAt,omitempty"`
+	LastHandledDeployAt string `json:"lastHandledDeployAt,omitempty"`
+
+	// +optional
+	LastHandledPruneAt string `json:"lastHandledPruneAt,omitempty"`
+
+	// +optional
+	LastHandledValidateAt string `json:"lastHandledValidateAt,omitempty"`
 
 	// ObservedGeneration is the last reconciled generation.
 	// +optional
@@ -319,25 +367,34 @@ type KluctlDeploymentStatus struct {
 	// +optional
 	LastObjectsHash string `json:"lastObjectsHash,omitempty"`
 
+	// +optional
+	LastManualObjectsHash *string `json:"lastManualObjectsHash,omitempty"`
+
+	// +optional
+	LastPrepareError string `json:"lastPrepareError,omitempty"`
 	// +optional
 	LastDeployError string `json:"lastDeployError,omitempty"`
 	// +optional
-	LastPruneError string `json:"lastPruneError,omitempty"`
-	// +optional
 	LastValidateError string `json:"lastValidateError,omitempty"`
+	// +optional
+	LastDriftDetectionError string `json:"lastDriftDetectionError,omitempty"`
 
-	// LastDeployResult is the result of the last deploy command
+	// LastDeployResult is the result summary of the last deploy command
 	// +optional
 	// +kubebuilder:pruning:PreserveUnknownFields
 	LastDeployResult *runtime.RawExtension `json:"lastDeployResult,omitempty"`
 
-	// LastDeployResult is the result of the last prune command
-	// +optional
-	LastPruneResult *runtime.RawExtension `json:"lastPruneResult,omitempty"`
-
-	// LastValidateResult is the result of the last validate command
+	// LastValidateResult is the result summary of the last validate command
 	// +optional
 	LastValidateResult *runtime.RawExtension `json:"lastValidateResult,omitempty"`
+
+	// LastDriftDetectionResult is the result of the last drift detection command
+	// optional
+	LastDriftDetectionResult *runtime.RawExtension `json:"lastDriftDetectionResult,omitempty"`
+
+	// LastDriftDetectionResultMessage contains a short message that describes the drift
+	// optional
+	LastDriftDetectionResultMessage string `json:"lastDriftDetectionResultMessage,omitempty"`
 }
 
 func (s *KluctlDeploymentStatus) SetLastDeployResult(crs *result.CommandResultSummary, err error) error {
@@ -357,23 +414,6 @@ func (s *KluctlDeploymentStatus) SetLastDeployResult(crs *result.CommandResultSu
 	return nil
 }
 
-func (s *KluctlDeploymentStatus) SetLastPruneResult(crs *result.CommandResultSummary, err error) error {
-	s.LastPruneError = ""
-	if err != nil {
-		s.LastPruneError = err.Error()
-	}
-	if crs == nil {
-		s.LastPruneResult = nil
-	} else {
-		b, err := yaml.WriteJsonString(crs)
-		if err != nil {
-			return err
-		}
-		s.LastPruneResult = &runtime.RawExtension{Raw: []byte(b)}
-	}
-	return nil
-}
-
 func (s *KluctlDeploymentStatus) SetLastValidateResult(crs *result.ValidateResult, err error) error {
 	s.LastValidateError = ""
 	if err != nil {
@@ -391,6 +431,25 @@ func (s *KluctlDeploymentStatus) SetLastValidateResult(crs *result.ValidateResul
 	return nil
 }
 
+func (s *KluctlDeploymentStatus) SetLastDriftDetectionResult(dr *result.DriftDetectionResult, err error) error {
+	s.LastDriftDetectionError = ""
+	s.LastDriftDetectionResultMessage = ""
+	if err != nil {
+		s.LastDriftDetectionError = err.Error()
+	}
+	if dr == nil {
+		s.LastDriftDetectionResult = nil
+	} else {
+		b, err := yaml.WriteJsonString(dr)
+		if err != nil {
+			return err
+		}
+		s.LastDriftDetectionResult = &runtime.RawExtension{Raw: []byte(b)}
+		s.LastDriftDetectionResultMessage = dr.BuildShortMessage()
+	}
+	return nil
+}
+
 func (s *KluctlDeploymentStatus) GetLastDeployResult() (*result.CommandResultSummary, error) {
 	if s.LastDeployResult == nil {
 		return nil, nil
@@ -403,18 +462,6 @@ func (s *KluctlDeploymentStatus) GetLastDeployResult() (*result.CommandResultSum
 	return &ret, nil
 }
 
-func (s *KluctlDeploymentStatus) GetLastPruneResult() (*result.CommandResultSummary, error) {
-	if s.LastPruneResult == nil {
-		return nil, nil
-	}
-	var ret result.CommandResultSummary
-	err := yaml.ReadYamlBytes(s.LastPruneResult.Raw, &ret)
-	if err != nil {
-		return nil, err
-	}
-	return &ret, nil
-}
-
 func (s *KluctlDeploymentStatus) GetLastValidateResult() (*result.ValidateResult, error) {
 	if s.LastValidateResult == nil {
 		return nil, nil
@@ -427,11 +474,24 @@ func (s *KluctlDeploymentStatus) GetLastValidateResult() (*result.ValidateResult
 	return &ret, nil
 }
 
+func (s *KluctlDeploymentStatus) GetDriftDetectionResult() (*result.DriftDetectionResult, error) {
+	if s.LastDriftDetectionResult == nil {
+		return nil, nil
+	}
+	var ret result.DriftDetectionResult
+	err := yaml.ReadYamlBytes(s.LastDriftDetectionResult.Raw, &ret)
+	if err != nil {
+		return nil, err
+	}
+	return &ret, nil
+}
+
 //+kubebuilder:object:root=true
 //+kubebuilder:subresource:status
 //+kubebuilder:printcolumn:name="DryRun",type="boolean",JSONPath=".spec.dryRun",description=""
 //+kubebuilder:printcolumn:name="Deployed",type="date",JSONPath=".status.lastDeployResult.commandInfo.endTime",description=""
 //+kubebuilder:printcolumn:name="Ready",type="string",JSONPath=".status.conditions[?(@.type==\"Ready\")].status",description=""
+//+kubebuilder:printcolumn:name="Drift",type="string",JSONPath=".status.lastDriftDetectionResultMessage",description=""
 //+kubebuilder:printcolumn:name="Status",type="string",JSONPath=".status.conditions[?(@.type==\"Ready\")].message",description=""
 //+kubebuilder:printcolumn:name="Age",type="date",JSONPath=".metadata.creationTimestamp",description=""
 
