@@ -3,24 +3,29 @@ package commands
 import (
 	"context"
 	"fmt"
+	"github.com/google/uuid"
 	"github.com/kluctl/kluctl/v2/cmd/kluctl/args"
 	"github.com/kluctl/kluctl/v2/pkg/deployment"
 	"github.com/kluctl/kluctl/v2/pkg/git"
 	"github.com/kluctl/kluctl/v2/pkg/git/auth"
 	"github.com/kluctl/kluctl/v2/pkg/git/messages"
 	ssh_pool "github.com/kluctl/kluctl/v2/pkg/git/ssh-pool"
+	"github.com/kluctl/kluctl/v2/pkg/k8s"
 	"github.com/kluctl/kluctl/v2/pkg/kluctl_jinja2"
 	"github.com/kluctl/kluctl/v2/pkg/kluctl_project"
+	"github.com/kluctl/kluctl/v2/pkg/prompts"
 	"github.com/kluctl/kluctl/v2/pkg/repocache"
 	"github.com/kluctl/kluctl/v2/pkg/results"
 	"github.com/kluctl/kluctl/v2/pkg/status"
 	"github.com/kluctl/kluctl/v2/pkg/types"
 	"github.com/kluctl/kluctl/v2/pkg/utils"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
+	"k8s.io/apimachinery/pkg/api/meta"
 	"k8s.io/client-go/rest"
 	"k8s.io/client-go/tools/clientcmd"
 	"k8s.io/client-go/tools/clientcmd/api"
 	"os"
+	client2 "sigs.k8s.io/controller-runtime/pkg/client"
 	"strings"
 )
 
@@ -61,14 +66,14 @@ func withKluctlProjectFromArgs(ctx context.Context, projectFlags args.ProjectFla
 
 	var repoOverrides []repocache.RepoOverride
 	for _, x := range projectFlags.LocalGitOverride {
-		ro, err := parseRepoOverride(x, false)
+		ro, err := parseRepoOverride(ctx, x, false)
 		if err != nil {
 			return fmt.Errorf("invalid --local-git-override: %w", err)
 		}
 		repoOverrides = append(repoOverrides, ro)
 	}
 	for _, x := range projectFlags.LocalGitGroupOverride {
-		ro, err := parseRepoOverride(x, true)
+		ro, err := parseRepoOverride(ctx, x, true)
 		if err != nil {
 			return fmt.Errorf("invalid --local-git-group-override: %w", err)
 		}
@@ -78,8 +83,8 @@ func withKluctlProjectFromArgs(ctx context.Context, projectFlags args.ProjectFla
 	messageCallbacks := &messages.MessageCallbacks{
 		WarningFn:            func(s string) { status.Warning(ctx, s) },
 		TraceFn:              func(s string) { status.Trace(ctx, s) },
-		AskForPasswordFn:     func(s string) (string, error) { return status.AskForPassword(ctx, s) },
-		AskForConfirmationFn: func(s string) bool { return status.AskForConfirmation(ctx, s) },
+		AskForPasswordFn:     func(s string) (string, error) { return prompts.AskForPassword(ctx, s) },
+		AskForConfirmationFn: func(s string) bool { return prompts.AskForConfirmation(ctx, s) },
 	}
 	gitAuth := auth.NewDefaultAuthProviders("KLUCTL_GIT", messageCallbacks)
 
@@ -141,9 +146,11 @@ type projectTargetCommandArgs struct {
 }
 
 type commandCtx struct {
-	ctx         context.Context
-	targetCtx   *kluctl_project.TargetContext
-	images      *deployment.Images
+	ctx       context.Context
+	targetCtx *kluctl_project.TargetContext
+	images    *deployment.Images
+
+	resultId    string
 	resultStore results.ResultStore
 }
 
@@ -193,7 +200,36 @@ func withProjectTargetCommandContext(ctx context.Context, args projectTargetComm
 		RenderOutputDir:    renderOutputDir,
 	}
 
-	targetCtx, err := p.NewTargetContext(ctx, targetParams)
+	commandResultId := uuid.NewString()
+
+	clientConfig, contextName, err := p.LoadK8sConfig(ctx, targetParams)
+	if err != nil {
+		return err
+	}
+
+	var k *k8s.K8sCluster
+	var resultStore results.ResultStore
+	if clientConfig != nil {
+		discovery, mapper, err := k8s.CreateDiscoveryAndMapper(ctx, clientConfig)
+		if err != nil {
+			return err
+		}
+
+		s := status.Start(ctx, fmt.Sprintf("Initializing k8s client"))
+		k, err = k8s.NewK8sCluster(ctx, clientConfig, discovery, mapper, targetParams.DryRun)
+		if err != nil {
+			s.Failed()
+			return err
+		}
+		s.Success()
+
+		resultStore, err = initStores(ctx, clientConfig, mapper, args.commandResultFlags)
+		if err != nil {
+			return err
+		}
+	}
+
+	targetCtx, err := p.NewTargetContext(ctx, contextName, k, targetParams)
 	if err != nil {
 		return err
 	}
@@ -204,30 +240,37 @@ func withProjectTargetCommandContext(ctx context.Context, args projectTargetComm
 			return err
 		}
 	}
-
-	var resultStore results.ResultStore
-	if args.commandResultFlags != nil && args.commandResultFlags.WriteCommandResult {
-		client, err := targetCtx.SharedContext.K.ToClient()
-		if err != nil {
-			return err
-		}
-
-		resultStore, err = results.NewResultStoreSecrets(ctx, client, nil, args.commandResultFlags.CommandResultNamespace, args.commandResultFlags.KeepCommandResultsCount)
-		if err != nil {
-			return err
-		}
-	}
-
 	cmdCtx := &commandCtx{
 		ctx:         ctx,
 		targetCtx:   targetCtx,
 		images:      images,
+		resultId:    commandResultId,
 		resultStore: resultStore,
 	}
 
 	return cb(cmdCtx)
 }
 
+func initStores(ctx context.Context, clientConfig *rest.Config, mapper meta.RESTMapper, flags *args.CommandResultFlags) (results.ResultStore, error) {
+	if flags == nil || !flags.WriteCommandResult {
+		return nil, nil
+	}
+
+	client, err := client2.New(clientConfig, client2.Options{
+		Mapper: mapper,
+	})
+	if err != nil {
+		return nil, err
+	}
+
+	resultStore, err := results.NewResultStoreSecrets(ctx, clientConfig, client, flags.CommandResultNamespace, flags.KeepCommandResultsCount, flags.KeepValidateResultsCount)
+	if err != nil {
+		return nil, err
+	}
+
+	return resultStore, nil
+}
+
 func clientConfigGetter(forCompletion bool) func(context *string) (*rest.Config, *api.Config, error) {
 	return func(context *string) (*rest.Config, *api.Config, error) {
 		if forCompletion {
@@ -255,9 +298,7 @@ func clientConfigGetter(forCompletion bool) func(context *string) (*rest.Config,
 	}
 }
 
-func parseRepoOverride(s string, isGroup bool) (ret repocache.RepoOverride, err error) {
-	ret.IsGroup = isGroup
-
+func parseRepoOverride(ctx context.Context, s string, isGroup bool) (repocache.RepoOverride, error) {
 	sp := strings.SplitN(s, "=", 2)
 	if len(sp) != 2 {
 		return repocache.RepoOverride{}, fmt.Errorf("%s", s)
@@ -265,10 +306,30 @@ func parseRepoOverride(s string, isGroup bool) (ret repocache.RepoOverride, err
 
 	repoKey, err := types.ParseGitRepoKey(sp[0])
 	if err != nil {
-		return repocache.RepoOverride{}, err
+		// try as legacy repo key
+		u, err2 := types.ParseGitUrl(sp[0])
+		if err2 != nil {
+			// return original error
+			return repocache.RepoOverride{}, err
+		}
+
+		x := u.Host
+		if !strings.HasPrefix(u.Path, "/") {
+			x += "/"
+		}
+		x += u.Path
+		repoKey, err2 = types.ParseGitRepoKey(x)
+		if err2 != nil {
+			// return original error
+			return repocache.RepoOverride{}, err
+		}
+
+		status.Deprecation(ctx, "old-repo-override", "Passing --local-git-override/--local-git-override-group in the example.com:path form is deprecated and will not be supported in future versions of Kluctl. Please use the example.com/path form.")
 	}
 
-	ret.RepoKey = repoKey
-	ret.Override = sp[1]
-	return
+	return repocache.RepoOverride{
+		RepoKey:  repoKey,
+		IsGroup:  isGroup,
+		Override: sp[1],
+	}, nil
 }
