@@ -6,6 +6,7 @@ import (
 	"fmt"
 	kluctlv1 "github.com/kluctl/kluctl/v2/api/v1beta1"
 	test_utils "github.com/kluctl/kluctl/v2/e2e/test-utils"
+	"github.com/kluctl/kluctl/v2/e2e/test_project"
 	types2 "github.com/kluctl/kluctl/v2/pkg/types"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
 	"github.com/onsi/gomega"
@@ -14,9 +15,11 @@ import (
 	"k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	"k8s.io/klog/v2"
 	"math/rand"
 	"os"
 	"path/filepath"
+	ctrl "sigs.k8s.io/controller-runtime"
 	"sigs.k8s.io/controller-runtime/pkg/client"
 	"strings"
 	"testing"
@@ -30,6 +33,11 @@ const (
 	interval = time.Second * 5
 )
 
+func init() {
+	// this must be called in the first 30 seconds of startup, so we have to do it here at init() time
+	ctrl.SetLogger(klog.NewKlogr())
+}
+
 type GitopsTestSuite struct {
 	suite.Suite
 
@@ -101,11 +109,10 @@ func (suite *GitopsTestSuite) startController() {
 		tmpKubeconfig,
 		"--context",
 		"context1",
-		"--write-command-result",
 	}
 	done := make(chan struct{})
 	go func() {
-		_, _, err := test_utils.KluctlExecute(suite.T(), ctx, args...)
+		_, _, err := test_project.KluctlExecute(suite.T(), ctx, args...)
 		if err != nil {
 			suite.T().Error(err)
 		}
@@ -158,7 +165,7 @@ func (suite *GitopsTestSuite) waitForCommit(key client.ObjectKey, commit string)
 	}, timeout, time.Second).Should(BeTrue())
 }
 
-func (suite *GitopsTestSuite) createKluctlDeployment(p *test_utils.TestProject, target string, args map[string]any) client.ObjectKey {
+func (suite *GitopsTestSuite) createKluctlDeployment(p *test_project.TestProject, target string, args map[string]any) client.ObjectKey {
 	gitopsNs := p.TestSlug() + "-gitops"
 	createNamespace(suite.T(), suite.k, gitopsNs)
 
@@ -176,7 +183,7 @@ func (suite *GitopsTestSuite) createKluctlDeployment(p *test_utils.TestProject,
 			Interval: metav1.Duration{Duration: interval},
 			Timeout:  &metav1.Duration{Duration: timeout},
 			Target:   &target,
-			Args: runtime.RawExtension{
+			Args: &runtime.RawExtension{
 				Raw: jargs,
 			},
 			Source: kluctlv1.ProjectSource{
@@ -227,11 +234,11 @@ func (suite *GitopsTestSuite) deleteKluctlDeployment(key client.ObjectKey) {
 func (suite *GitopsTestSuite) TestGitOpsFieldManager() {
 	g := NewWithT(suite.T())
 
-	p := test_utils.NewTestProject(suite.T())
+	p := test_project.NewTestProject(suite.T())
 	createNamespace(suite.T(), suite.k, p.TestSlug())
 
 	p.UpdateTarget("target1", nil)
-	p.AddKustomizeDeployment("d1", []test_utils.KustomizeResource{
+	p.AddKustomizeDeployment("d1", []test_project.KustomizeResource{
 		{Name: "cm1.yaml", Content: uo.FromStringMust(`apiVersion: v1
 kind: ConfigMap
 metadata:
@@ -345,7 +352,7 @@ data:
 func (suite *GitopsTestSuite) TestKluctlDeploymentReconciler_Helm() {
 	g := NewWithT(suite.T())
 
-	p := test_utils.NewTestProject(suite.T())
+	p := test_project.NewTestProject(suite.T())
 	createNamespace(suite.T(), suite.k, p.TestSlug())
 
 	p.UpdateTarget("target1", nil)
@@ -509,12 +516,12 @@ func (suite *GitopsTestSuite) TestKluctlDeploymentReconciler_Helm() {
 func (suite *GitopsTestSuite) TestKluctlDeploymentReconciler_Prune() {
 	g := NewWithT(suite.T())
 
-	p := test_utils.NewTestProject(suite.T())
+	p := test_project.NewTestProject(suite.T())
 	createNamespace(suite.T(), suite.k, p.TestSlug())
 
 	p.UpdateTarget("target1", nil)
 
-	p.AddKustomizeDeployment("d1", []test_utils.KustomizeResource{
+	p.AddKustomizeDeployment("d1", []test_project.KustomizeResource{
 		{Name: "cm1.yaml", Content: uo.FromStringMust(`apiVersion: v1
 kind: ConfigMap
 metadata:
@@ -524,7 +531,7 @@ data:
   k1: v1
 `)},
 	}, nil)
-	p.AddKustomizeDeployment("d2", []test_utils.KustomizeResource{
+	p.AddKustomizeDeployment("d2", []test_project.KustomizeResource{
 		{Name: "cm2.yaml", Content: uo.FromStringMust(`apiVersion: v1
 kind: ConfigMap
 metadata:
@@ -608,12 +615,12 @@ data:
 func (suite *GitopsTestSuite) doTestDelete(delete bool) {
 	g := NewWithT(suite.T())
 
-	p := test_utils.NewTestProject(suite.T())
+	p := test_project.NewTestProject(suite.T())
 	createNamespace(suite.T(), suite.k, p.TestSlug())
 
 	p.UpdateTarget("target1", nil)
 
-	p.AddKustomizeDeployment("d1", []test_utils.KustomizeResource{
+	p.AddKustomizeDeployment("d1", []test_project.KustomizeResource{
 		{Name: "cm1.yaml", Content: uo.FromStringMust(`apiVersion: v1
 kind: ConfigMap
 metadata:
