@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"github.com/bitnami-labs/sealed-secrets/pkg/crypto"
 	"github.com/kluctl/kluctl/v2/e2e/test-utils"
+	"github.com/kluctl/kluctl/v2/e2e/test_project"
 	"github.com/kluctl/kluctl/v2/e2e/test_resources"
 	"github.com/kluctl/kluctl/v2/pkg/seal"
 	"github.com/kluctl/kluctl/v2/pkg/utils"
@@ -85,7 +86,7 @@ func startCertServer() (*certServer, error) {
 	return &cs, nil
 }
 
-func addProxyVars(p *test_utils.TestProject) {
+func addProxyVars(p *test_project.TestProject) {
 	f := func(idx int, k *test_utils.EnvTestCluster, cs *certServer) {
 		p.AddExtraEnv(fmt.Sprintf("KLUCTL_K8S_SERVICE_PROXY_%d_API_HOST=%s", idx, k.RESTConfig().Host))
 		p.AddExtraEnv(fmt.Sprintf("KLUCTL_K8S_SERVICE_PROXY_%d_SERVICE_NAMESPACE=%s", idx, "kube-system"))
@@ -97,8 +98,8 @@ func addProxyVars(p *test_utils.TestProject) {
 	f(1, defaultCluster2, certServer2)
 }
 
-func prepareSealTest(t *testing.T, k *test_utils.EnvTestCluster, secrets map[string]string, varsSources []*uo.UnstructuredObject, proxy bool) *test_utils.TestProject {
-	p := test_utils.NewTestProject(t, test_utils.WithUseProcess(true))
+func prepareSealTest(t *testing.T, k *test_utils.EnvTestCluster, secrets map[string]string, varsSources []*uo.UnstructuredObject, proxy bool) *test_project.TestProject {
+	p := test_project.NewTestProject(t, test_project.WithUseProcess(true))
 
 	if proxy {
 		addProxyVars(p)
@@ -114,13 +115,13 @@ func prepareSealTest(t *testing.T, k *test_utils.EnvTestCluster, secrets map[str
 	return p
 }
 
-func addSecretsSet(p *test_utils.TestProject, name string, varsSources []*uo.UnstructuredObject) {
+func addSecretsSet(p *test_project.TestProject, name string, varsSources []*uo.UnstructuredObject) {
 	p.UpdateSecretSet(name, func(secretSet *uo.UnstructuredObject) {
 		_ = secretSet.SetNestedField(varsSources, "vars")
 	})
 }
 
-func addSecretsSetToTarget(p *test_utils.TestProject, targetName string, secretSetName string) {
+func addSecretsSetToTarget(p *test_project.TestProject, targetName string, secretSetName string) {
 	p.UpdateTarget(targetName, func(target *uo.UnstructuredObject) {
 		l, _, _ := target.GetNestedList("sealingConfig", "secretSets")
 		l = append(l, secretSetName)
