@@ -4,20 +4,22 @@ import (
 	"context"
 	errors2 "errors"
 	"fmt"
-	"github.com/hashicorp/go-retryablehttp"
+	"github.com/google/uuid"
+	"github.com/hashicorp/go-multierror"
 	kluctlv1 "github.com/kluctl/kluctl/v2/api/v1beta1"
 	internal_metrics "github.com/kluctl/kluctl/v2/pkg/controllers/metrics"
 	ssh_pool "github.com/kluctl/kluctl/v2/pkg/git/ssh-pool"
 	"github.com/kluctl/kluctl/v2/pkg/kluctl_jinja2"
 	"github.com/kluctl/kluctl/v2/pkg/results"
 	"github.com/kluctl/kluctl/v2/pkg/status"
+	"github.com/kluctl/kluctl/v2/pkg/types/k8s"
 	"github.com/kluctl/kluctl/v2/pkg/types/result"
+	"github.com/kluctl/kluctl/v2/pkg/utils"
 	"github.com/kluctl/kluctl/v2/pkg/utils/flux_utils/meta"
 	"github.com/kluctl/kluctl/v2/pkg/utils/flux_utils/metrics"
 	"github.com/kluctl/kluctl/v2/pkg/yaml"
-	log "github.com/sirupsen/logrus"
 	"k8s.io/apimachinery/pkg/api/errors"
-	meta2 "k8s.io/apimachinery/pkg/api/meta"
+	apimeta "k8s.io/apimachinery/pkg/api/meta"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -25,19 +27,20 @@ import (
 	"k8s.io/client-go/discovery"
 	"k8s.io/client-go/rest"
 	kuberecorder "k8s.io/client-go/tools/record"
-	"k8s.io/client-go/tools/reference"
 	"path"
+	"reflect"
 	ctrl "sigs.k8s.io/controller-runtime"
 	"sigs.k8s.io/controller-runtime/pkg/client"
+	"sigs.k8s.io/controller-runtime/pkg/client/apiutil"
+	"sigs.k8s.io/controller-runtime/pkg/controller"
 	"sigs.k8s.io/controller-runtime/pkg/controller/controllerutil"
+	"sync"
 	"time"
 )
 
 type KluctlDeploymentReconciler struct {
 	client.Client
 	RestConfig            *rest.Config
-	httpClient            *retryablehttp.Client
-	requeueDependency     time.Duration
 	Scheme                *runtime.Scheme
 	EventRecorder         kuberecorder.EventRecorder
 	MetricsRecorder       *metrics.Recorder
@@ -48,11 +51,14 @@ type KluctlDeploymentReconciler struct {
 	SshPool *ssh_pool.SshPool
 
 	ResultStore results.ResultStore
+
+	mutex               sync.Mutex
+	resourceVersionsMap map[client.ObjectKey]map[k8s.ObjectRef]string
 }
 
 // KluctlDeploymentReconcilerOpts contains options for the BaseReconciler.
 type KluctlDeploymentReconcilerOpts struct {
-	HTTPRetry int
+	Concurrency int
 }
 
 // +kubebuilder:rbac:groups=gitops.kluctl.io,resources=kluctldeployments,verbs=get;list;watch;create;update;patch;delete
@@ -66,24 +72,28 @@ type KluctlDeploymentReconcilerOpts struct {
 func (r *KluctlDeploymentReconciler) Reconcile(ctx context.Context, req ctrl.Request) (ctrl.Result, error) {
 	reconcileStart := time.Now()
 
-	ctx = status.NewContext(ctx, status.NewSimpleStatusHandler(func(message string) {
+	// we reuse the reconcileID as command result id
+	reconcileID := string(controller.ReconcileIDFromContext(ctx))
+
+	log := ctrl.LoggerFrom(ctx)
+
+	ctx = status.NewContext(ctx, status.NewSimpleStatusHandler(func(level status.Level, message string) {
 		log.Info(message)
-	}, false, false))
+	}, false))
 
 	obj := &kluctlv1.KluctlDeployment{}
 	if err := r.Get(ctx, req.NamespacedName, obj); err != nil {
 		return ctrl.Result{}, client.IgnoreNotFound(err)
 	}
 
-	var cancel context.CancelFunc
-	ctx, cancel = context.WithTimeout(ctx, r.calcTimeout(obj))
-	defer cancel()
-
 	retryInterval := obj.Spec.GetRetryInterval()
 	interval := obj.Spec.Interval.Duration
 
-	// Record suspended status metric
-	defer r.recordSuspension(ctx, obj)
+	defer func() {
+		r.recordSuspension(ctx, obj)
+		r.recordReadiness(ctx, obj)
+		r.recordDuration(ctx, obj, reconcileStart)
+	}()
 
 	// Add our finalizer if it does not exist
 	if !controllerutil.ContainsFinalizer(obj, kluctlv1.KluctlDeploymentFinalizer) {
@@ -93,6 +103,7 @@ func (r *KluctlDeploymentReconciler) Reconcile(ctx context.Context, req ctrl.Req
 			log.Error(err, "unable to register finalizer")
 			return ctrl.Result{}, err
 		}
+		return ctrl.Result{Requeue: true}, nil
 	}
 
 	// Examine if the object is under deletion
@@ -106,24 +117,23 @@ func (r *KluctlDeploymentReconciler) Reconcile(ctx context.Context, req ctrl.Req
 		return ctrl.Result{}, nil
 	}
 
-	// record reconciliation duration
-	if r.MetricsRecorder != nil {
-		objRef, err := reference.GetReference(r.Scheme, obj)
-		if err != nil {
-			return ctrl.Result{}, err
-		}
-		defer r.MetricsRecorder.RecordDuration(*objRef, reconcileStart)
+	patchObj := obj.DeepCopy()
+	patch := client.MergeFrom(patchObj)
+
+	// reconcile kluctlDeployment by applying the latest revision
+	ctrlResult, reconcileErr := r.doReconcile(ctx, obj, reconcileID)
+
+	// make sure conditions were not touched in the obj. changing conditions is only allowed via patchCondition,
+	// which directly patches the object in-cluster and does not touch the local obj
+	if !reflect.DeepEqual(patchObj.GetConditions(), obj.GetConditions()) {
+		panic("conditions were modified in doReconcile")
 	}
 
-	patch := client.MergeFrom(obj.DeepCopy())
-	// reconcile kluctlDeployment by applying the latest revision
-	ctrlResult, reconcileErr := r.doReconcile(ctx, obj)
+	// now patch all the other changes to the status
 	if err := r.Status().Patch(ctx, obj, patch, client.FieldOwner(r.ControllerName)); err != nil {
 		return ctrl.Result{Requeue: true}, err
 	}
 
-	r.recordReadiness(ctx, obj)
-
 	if ctrlResult == nil {
 		if reconcileErr != nil {
 			ctrlResult = &ctrl.Result{RequeueAfter: retryInterval}
@@ -155,22 +165,55 @@ func (r *KluctlDeploymentReconciler) Reconcile(ctx context.Context, req ctrl.Req
 
 func (r *KluctlDeploymentReconciler) doReconcile(
 	ctx context.Context,
-	obj *kluctlv1.KluctlDeployment) (*ctrl.Result, error) {
+	obj *kluctlv1.KluctlDeployment,
+	reconcileId string) (*ctrl.Result, error) {
 
 	log := ctrl.LoggerFrom(ctx)
+	key := client.ObjectKeyFromObject(obj)
 
 	r.exportDeploymentObjectToProm(obj)
 
+	// keep old status until we're doing real work (deploying, validating, ...)
+	oldReadyCondition := apimeta.FindStatusCondition(obj.GetConditions(), meta.ReadyCondition)
+
 	doFail := func(reason string, err error) (*ctrl.Result, error) {
-		setReadiness(obj, metav1.ConditionFalse, reason, err.Error())
 		internal_metrics.NewKluctlLastObjectStatus(obj.Namespace, obj.Name).Set(0.0)
+		patchErr := r.patchCondition(ctx, key, func(c *[]metav1.Condition) error {
+			setReadinessCondition(c, metav1.ConditionFalse, reason, err.Error(), obj.Generation)
+			apimeta.RemoveStatusCondition(c, meta.ReconcilingCondition)
+			return nil
+		})
+		if patchErr != nil {
+			err = multierror.Append(err, patchErr)
+		}
 		return nil, err
 	}
 
 	doFailPrepare := func(err error) (*ctrl.Result, error) {
+		obj.Status.LastPrepareError = err.Error()
 		return doFail(kluctlv1.PrepareFailedReason, err)
 	}
 
+	doProgressingCondition := func(message string, keepOldReadyStatus bool) error {
+		return r.patchCondition(ctx, key, func(c *[]metav1.Condition) error {
+			if keepOldReadyStatus && oldReadyCondition != nil {
+				setReadinessCondition(c, oldReadyCondition.Status, oldReadyCondition.Reason, oldReadyCondition.Message, obj.Generation)
+			} else {
+				setReadinessCondition(c, metav1.ConditionUnknown, meta.ProgressingReason, "Reconciliation in progress", obj.Generation)
+			}
+			setReconcilingCondition(c, metav1.ConditionTrue, meta.ProgressingReason, message, obj.Generation)
+			return nil
+		})
+	}
+
+	doReadyCondition := func(status metav1.ConditionStatus, reason, message string) error {
+		return r.patchCondition(ctx, key, func(c *[]metav1.Condition) error {
+			setReadinessCondition(c, status, reason, message, obj.Generation)
+			apimeta.RemoveStatusCondition(c, meta.ReconcilingCondition)
+			return nil
+		})
+	}
+
 	// record the value of the reconciliation request, if any
 	if v, ok := obj.GetAnnotations()[kluctlv1.KluctlRequestReconcileAnnotation]; ok {
 		obj.Status.LastHandledReconcileAt = v
@@ -181,45 +224,86 @@ func (r *KluctlDeploymentReconciler) doReconcile(
 		return nil, err
 	}
 	if legacyKd {
-		c := meta2.FindStatusCondition(obj.Status.Conditions, meta.ReadyCondition)
+		c := apimeta.FindStatusCondition(obj.Status.Conditions, meta.ReadyCondition)
 		if c != nil && c.Reason == kluctlv1.WaitingForLegacyMigrationReason {
 			log.Info("legacy KluctlDeployment does not have the readyForMigration status set. Skipping reconciliation. ")
 			return &ctrl.Result{RequeueAfter: 5 * time.Second}, err
 		}
 		log.Info("legacy KluctlDeployment does not have the readyForMigration status set. Skipping reconciliation. " +
 			"Please ensure that you have upgraded to the latest version of the legacy flux-kluctl-controller and that is is still running.")
-		setReadiness(obj, metav1.ConditionFalse, kluctlv1.WaitingForLegacyMigrationReason, "waiting for the legacy controller to set readyForMigration=true")
-		r.recordReadiness(ctx, obj)
+		patchErr := doReadyCondition(metav1.ConditionFalse, kluctlv1.WaitingForLegacyMigrationReason, "waiting for the legacy controller to set readyForMigration=true")
+		if patchErr != nil {
+			return nil, patchErr
+		}
 		return &ctrl.Result{Requeue: true}, nil
 	} else {
-		c := meta2.FindStatusCondition(obj.Status.Conditions, meta.ReadyCondition)
+		c := apimeta.FindStatusCondition(obj.Status.Conditions, meta.ReadyCondition)
 		if c != nil && c.Reason == kluctlv1.WaitingForLegacyMigrationReason {
 			log.Info("legacy KluctlDeployment has the readyForMigration status set now")
-			setReadiness(obj, metav1.ConditionFalse, meta.ProgressingReason, "migration is finished")
-			r.recordReadiness(ctx, obj)
+			patchErr := doReadyCondition(metav1.ConditionFalse, meta.ProgressingReason, "migration is finished")
+			if patchErr != nil {
+				return nil, patchErr
+			}
 			return &ctrl.Result{Requeue: true}, nil
 		}
 	}
 
-	// set the reconciliation status to progressing
-	if obj.Status.ObservedGeneration == 0 {
-		setReadiness(obj, metav1.ConditionUnknown, meta.ProgressingReason, "reconciliation in progress")
-		r.recordReadiness(ctx, obj)
+	patchErr := doProgressingCondition("Initializing", true)
+	if patchErr != nil {
+		return nil, patchErr
 	}
 
 	oldGeneration := obj.Status.ObservedGeneration
 	oldDeployRequest := obj.Status.LastHandledDeployAt
+	oldPruneRequest := obj.Status.LastHandledPruneAt
+	oldValidateRequest := obj.Status.LastHandledValidateAt
 	curDeployRequest, _ := obj.GetAnnotations()[kluctlv1.KluctlRequestDeployAnnotation]
+	curPruneRequest, _ := obj.GetAnnotations()[kluctlv1.KluctlRequestPruneAnnotation]
+	curValidateRequest, _ := obj.GetAnnotations()[kluctlv1.KluctlRequestValidateAnnotation]
 	obj.Status.ObservedGeneration = obj.GetGeneration()
 	obj.Status.LastHandledDeployAt = curDeployRequest
+	obj.Status.LastHandledPruneAt = curPruneRequest
+	obj.Status.LastHandledValidateAt = curValidateRequest
 
-	pp, err := prepareProject(ctx, r, obj, true)
+	patchErr = doProgressingCondition("Preparing project", true)
+	if patchErr != nil {
+		return nil, patchErr
+	}
+
+	obj.Status.LastPrepareError = ""
+
+	timeoutCtx, cancel := context.WithTimeout(ctx, r.calcTimeout(obj))
+	defer cancel()
+
+	pp, err := prepareProject(timeoutCtx, r, obj, true)
 	if err != nil {
 		return doFailPrepare(err)
 	}
 	defer pp.cleanup()
 
-	obj.Status.ObservedCommit = pp.commit
+	patchErr = doProgressingCondition("Loading project and target", true)
+	if patchErr != nil {
+		return nil, patchErr
+	}
+
+	obj.Status.ObservedCommit = pp.co.CheckedOutCommit
+
+	newProjectKey := result.ProjectKey{
+		GitRepoKey: obj.Spec.Source.URL.RepoKey(),
+		SubDir:     path.Clean(obj.Spec.Source.Path),
+	}
+	if newProjectKey.SubDir == "." {
+		newProjectKey.SubDir = ""
+	}
+
+	// we patch the projectKey immediately so that the webui knows it asap
+	if obj.Status.ProjectKey == nil || *obj.Status.ProjectKey != newProjectKey {
+		patchErr = r.patchStatus(ctx, key, func(status *kluctlv1.KluctlDeploymentStatus) error {
+			status.ProjectKey = &newProjectKey
+			return nil
+		})
+	}
+	obj.Status.ProjectKey = &newProjectKey
 
 	j2, err := kluctl_jinja2.NewKluctlJinja2(true)
 	if err != nil {
@@ -229,44 +313,56 @@ func (r *KluctlDeploymentReconciler) doReconcile(
 
 	pt := pp.newTarget()
 
-	lp, err := pp.loadKluctlProject(ctx, pt, j2)
+	lp, err := pp.loadKluctlProject(timeoutCtx, pt, j2)
 	if err != nil {
 		return doFailPrepare(err)
 	}
 
-	targetContext, err := pt.loadTarget(ctx, lp)
+	targetContext, err := pt.loadTarget(timeoutCtx, lp)
+	if targetContext != nil {
+		clusterId, err := targetContext.SharedContext.K.GetClusterId()
+		if err != nil {
+			return doFailPrepare(err)
+		}
+		newTargetKey := result.TargetKey{
+			TargetName:    targetContext.Target.Name,
+			Discriminator: targetContext.Target.Discriminator,
+			ClusterId:     clusterId,
+		}
+		// we patch the targetKey immediately so that the webui knows it asap
+		if obj.Status.TargetKey == nil || *obj.Status.TargetKey != newTargetKey {
+			patchErr = r.patchStatus(ctx, key, func(status *kluctlv1.KluctlDeploymentStatus) error {
+				status.TargetKey = &newTargetKey
+				return nil
+			})
+		}
+		obj.Status.TargetKey = &newTargetKey
+	}
 	if err != nil {
 		return doFailPrepare(err)
 	}
 
-	obj.Status.ProjectKey = &result.ProjectKey{
-		GitRepoKey: obj.Spec.Source.URL.RepoKey(),
-		SubDir:     path.Clean(obj.Spec.Source.Path),
-	}
-	obj.Status.TargetKey = &result.TargetKey{
-		TargetName:    targetContext.Target.Name,
-		Discriminator: targetContext.Target.Discriminator,
-	}
-
-	if obj.Status.ProjectKey.SubDir == "." {
-		obj.Status.ProjectKey.SubDir = ""
-	}
-
 	objectsHash, err := targetContext.DeploymentCollection.CalcObjectsHash()
 	if err != nil {
 		return doFailPrepare(err)
 	}
 
 	needDeploy := false
+	needDeployByRequest := false
 	needPrune := false
 	needValidate := false
+	needDriftDetection := true
 
 	if obj.Status.LastDeployResult == nil || obj.Status.LastObjectsHash != objectsHash {
 		// either never deployed or source code changed
 		needDeploy = true
+	} else if obj.Spec.Manual && !utils.StrPtrEquals(obj.Status.LastManualObjectsHash, obj.Spec.ManualObjectsHash) {
+		// approval hash was changed
+		needDeploy = true
 	} else if curDeployRequest != "" && oldDeployRequest != curDeployRequest {
 		// explicitly requested a deploy
 		needDeploy = true
+		needDeployByRequest = true
 	} else if oldGeneration != obj.GetGeneration() {
 		// spec has changed
 		needDeploy = true
@@ -278,10 +374,28 @@ func (r *KluctlDeploymentReconciler) doReconcile(
 		}
 	}
 
+	if needDeploy && obj.Spec.Manual {
+		log.Info("checking manual object hash")
+		if obj.Spec.ManualObjectsHash == nil || *obj.Spec.ManualObjectsHash != objectsHash {
+			log.Info("deployment is not approved", "manualObjectsHash", obj.Spec.ManualObjectsHash, "objectsHash", objectsHash)
+			needDeploy = false
+		} else {
+			log.Info("deployment is approved", "objectsHash", objectsHash)
+		}
+	}
+
+	if curPruneRequest != "" && oldPruneRequest != curPruneRequest {
+		// explicitly requested a prune
+		needPrune = true
+	}
+
 	if obj.Spec.Validate {
 		if obj.Status.LastValidateResult == nil || needDeploy {
 			// either never validated before or a deployment requested (which required re-validation)
 			needValidate = true
+		} else if curValidateRequest != "" && oldValidateRequest != curValidateRequest {
+			// explicitly requested a validate
+			needValidate = true
 		} else {
 			nextValidateTime := r.nextValidateTime(obj)
 			if nextValidateTime != nil {
@@ -293,48 +407,79 @@ func (r *KluctlDeploymentReconciler) doReconcile(
 		obj.Status.LastValidateError = ""
 	}
 
-	if obj.Spec.Prune {
-		needPrune = needDeploy
-	} else {
-		obj.Status.LastPruneResult = nil
-		obj.Status.LastPruneError = ""
+	if !needDeploy && obj.Status.LastObjectsHash != objectsHash {
+		// force full drift detection as we can't know which objects changed in-between
+		r.updateResourceVersions(key, nil, nil)
 	}
 
 	obj.Status.LastObjectsHash = objectsHash
+	obj.Status.LastManualObjectsHash = obj.Spec.ManualObjectsHash
 
 	var deployResult *result.CommandResult
 	if needDeploy {
-		// deploy the kluctl project
-		if obj.Spec.DeployMode == kluctlv1.KluctlDeployModeFull {
-			deployResult, err = pt.kluctlDeploy(ctx, targetContext)
-		} else if obj.Spec.DeployMode == kluctlv1.KluctlDeployPokeImages {
-			deployResult, err = pt.kluctlPokeImages(ctx, targetContext)
-		} else {
-			err = fmt.Errorf("deployMode '%s' not supported", obj.Spec.DeployMode)
+		patchErr = doProgressingCondition(fmt.Sprintf("Performing kluctl %s", obj.Spec.DeployMode), false)
+		if patchErr != nil {
+			return nil, patchErr
 		}
+		deployResult, err = pt.kluctlDeployOrPokeImages(ctx, obj.Spec.DeployMode, targetContext, reconcileId, objectsHash, needDeployByRequest)
 		err = obj.Status.SetLastDeployResult(deployResult.BuildSummary(), err)
 		if err != nil {
 			log.Error(err, "Failed to write deploy result")
 		}
-	}
 
-	if needPrune {
-		// run garbage collection for stale objects that do not have pruning disabled
-		pruneResult, err := pt.kluctlPrune(ctx, targetContext)
-		err = obj.Status.SetLastPruneResult(pruneResult.BuildSummary(), err)
-		if err != nil {
-			log.Error(err, "Failed to write prune result")
+		if obj.Spec.DryRun {
+			// force full drift detection (otherwise we'd see the dry-run applied changes as non-drifted)
+			r.updateResourceVersions(key, nil, nil)
+		} else {
+			r.updateResourceVersions(key, deployResult.Objects, nil)
 		}
 	}
 
+	if needPrune {
+		patchErr = doProgressingCondition("Performing kluctl prune", false)
+		if patchErr != nil {
+			return nil, patchErr
+		}
+		pruneResult, err := pt.kluctlPrune(ctx, targetContext, reconcileId, objectsHash)
+		err = obj.Status.SetLastDeployResult(pruneResult.BuildSummary(), err)
+		if err != nil {
+			log.Error(err, "Failed to write prune result")
+		}
+
+		// force full drift detection
+		r.updateResourceVersions(key, nil, nil)
+	}
+
 	if needValidate {
-		validateResult, err := pt.kluctlValidate(ctx, targetContext, deployResult)
+		patchErr = doProgressingCondition("Performing kluctl validate", false)
+		if patchErr != nil {
+			return nil, patchErr
+		}
+		validateResult, err := pt.kluctlValidate(ctx, targetContext, deployResult, reconcileId, objectsHash)
 		err = obj.Status.SetLastValidateResult(validateResult, err)
 		if err != nil {
 			log.Error(err, "Failed to write validate result")
 		}
 	}
 
+	if needDriftDetection {
+		patchErr = doProgressingCondition("Performing drift detection", false)
+		if patchErr != nil {
+			return nil, patchErr
+		}
+
+		resourceVersions := r.getResourceVersions(key)
+
+		diffResult, cmdErr := pt.kluctlDiff(ctx, targetContext, reconcileId, objectsHash, resourceVersions)
+		driftDetectionResult := diffResult.BuildDriftDetectionResult()
+		err = obj.Status.SetLastDriftDetectionResult(driftDetectionResult, cmdErr)
+		if err != nil {
+			log.Error(err, "Failed to write drift detection result")
+		}
+
+		r.updateResourceVersions(key, diffResult.Objects, driftDetectionResult.Objects)
+	}
+
 	var ctrlResult ctrl.Result
 	ctrlResult.RequeueAfter = r.nextReconcileTime(obj).Sub(time.Now())
 	if ctrlResult.RequeueAfter < 0 {
@@ -344,15 +489,62 @@ func (r *KluctlDeploymentReconciler) doReconcile(
 
 	finalStatus, reason := r.buildFinalStatus(ctx, obj)
 	if reason != kluctlv1.ReconciliationSucceededReason {
-		setReadiness(obj, metav1.ConditionFalse, reason, finalStatus)
 		internal_metrics.NewKluctlLastObjectStatus(obj.Namespace, obj.Name).Set(0.0)
-		return &ctrlResult, fmt.Errorf(finalStatus)
+		err = fmt.Errorf(finalStatus)
+		patchErr = doReadyCondition(metav1.ConditionFalse, reason, finalStatus)
+		if patchErr != nil {
+			err = multierror.Append(err, patchErr)
+			return nil, err
+		}
+		return &ctrlResult, err
 	}
-	setReadiness(obj, metav1.ConditionTrue, reason, finalStatus)
 	internal_metrics.NewKluctlLastObjectStatus(obj.Namespace, obj.Name).Set(1.0)
+	patchErr = doReadyCondition(metav1.ConditionTrue, reason, finalStatus)
+	if patchErr != nil {
+		return nil, patchErr
+	}
 	return &ctrlResult, nil
 }
 
+func (r *KluctlDeploymentReconciler) buildResourceVersionsMap(objects []result.ResultObject) map[k8s.ObjectRef]string {
+	m := map[k8s.ObjectRef]string{}
+	for _, o := range objects {
+		if o.Applied == nil && o.Remote == nil {
+			continue
+		}
+		ro := o.Applied
+		if ro == nil {
+			ro = o.Remote
+		}
+		s := ro.GetK8sResourceVersion()
+		if s == "" {
+			continue
+		}
+		m[o.Ref] = s
+	}
+	return m
+}
+
+func (r *KluctlDeploymentReconciler) getResourceVersions(key client.ObjectKey) map[k8s.ObjectRef]string {
+	r.mutex.Lock()
+	defer r.mutex.Unlock()
+	m, _ := r.resourceVersionsMap[key]
+	return m
+}
+
+func (r *KluctlDeploymentReconciler) updateResourceVersions(key client.ObjectKey, diffObjects []result.ResultObject, driftedObjects []result.DriftedObject) {
+	newMap := r.buildResourceVersionsMap(diffObjects)
+
+	// ignore versions from already drifted objects so that we re-diff them the next time
+	for _, o := range driftedObjects {
+		delete(newMap, o.Ref)
+	}
+
+	r.mutex.Lock()
+	defer r.mutex.Unlock()
+	r.resourceVersionsMap[key] = newMap
+}
+
 func (r *KluctlDeploymentReconciler) buildFinalStatus(ctx context.Context, obj *kluctlv1.KluctlDeployment) (finalStatus string, reason string) {
 	log := ctrl.LoggerFrom(ctx)
 
@@ -360,10 +552,6 @@ func (r *KluctlDeploymentReconciler) buildFinalStatus(ctx context.Context, obj *
 		finalStatus = obj.Status.LastDeployError
 		reason = kluctlv1.DeployFailedReason
 		return
-	} else if obj.Status.LastPruneError != "" {
-		finalStatus = obj.Status.LastPruneError
-		reason = kluctlv1.PruneFailedReason
-		return
 	} else if obj.Status.LastValidateError != "" {
 		finalStatus = obj.Status.LastValidateError
 		reason = kluctlv1.ValidateFailedReason
@@ -371,7 +559,6 @@ func (r *KluctlDeploymentReconciler) buildFinalStatus(ctx context.Context, obj *
 	}
 
 	var lastDeployResult *result.CommandResultSummary
-	var lastPruneResult *result.CommandResultSummary
 	var lastValidateResult *result.ValidateResult
 	if obj.Status.LastDeployResult != nil {
 		err := yaml.ReadYamlBytes(obj.Status.LastDeployResult.Raw, &lastDeployResult)
@@ -379,12 +566,6 @@ func (r *KluctlDeploymentReconciler) buildFinalStatus(ctx context.Context, obj *
 			log.Info(fmt.Sprintf("Failed to parse last deploy result: %s", err.Error()))
 		}
 	}
-	if obj.Status.LastPruneResult != nil {
-		err := yaml.ReadYamlBytes(obj.Status.LastPruneResult.Raw, &lastPruneResult)
-		if err != nil {
-			log.Info(fmt.Sprintf("Failed to parse last prune result: %s", err.Error()))
-		}
-	}
 	if obj.Status.LastValidateResult != nil {
 		err := yaml.ReadYamlBytes(obj.Status.LastValidateResult.Raw, &lastValidateResult)
 		if err != nil {
@@ -393,12 +574,8 @@ func (r *KluctlDeploymentReconciler) buildFinalStatus(ctx context.Context, obj *
 	}
 
 	deployOk := lastDeployResult != nil && len(lastDeployResult.Errors) == 0
-	pruneOk := lastPruneResult != nil && len(lastPruneResult.Errors) == 0
 	validateOk := lastValidateResult != nil && len(lastValidateResult.Errors) == 0 && lastValidateResult.Ready
 
-	if !obj.Spec.Prune {
-		pruneOk = true
-	}
 	if !obj.Spec.Validate {
 		validateOk = true
 	}
@@ -411,17 +588,6 @@ func (r *KluctlDeploymentReconciler) buildFinalStatus(ctx context.Context, obj *
 			finalStatus += "failed"
 		}
 	}
-	if obj.Spec.Prune && obj.Status.LastPruneResult != nil {
-		if finalStatus != "" {
-			finalStatus += ", "
-		}
-		finalStatus += "prune: "
-		if pruneOk {
-			finalStatus += "ok"
-		} else {
-			finalStatus += "failed"
-		}
-	}
 	if obj.Spec.Validate && obj.Status.LastValidateResult != nil {
 		if finalStatus != "" {
 			finalStatus += ", "
@@ -434,7 +600,7 @@ func (r *KluctlDeploymentReconciler) buildFinalStatus(ctx context.Context, obj *
 		}
 	}
 
-	if deployOk && pruneOk {
+	if deployOk {
 		if validateOk {
 			reason = kluctlv1.ReconciliationSucceededReason
 		} else {
@@ -519,8 +685,9 @@ func (r *KluctlDeploymentReconciler) nextValidateTime(obj *kluctlv1.KluctlDeploy
 func (r *KluctlDeploymentReconciler) finalize(ctx context.Context, obj *kluctlv1.KluctlDeployment) (ctrl.Result, error) {
 	r.doFinalize(ctx, obj)
 
-	// Record deleted status
-	r.recordReadiness(ctx, obj)
+	r.mutex.Lock()
+	delete(r.resourceVersionsMap, client.ObjectKeyFromObject(obj))
+	r.mutex.Unlock()
 
 	// Remove our finalizer from the list and update it
 	patch := client.MergeFrom(obj.DeepCopy())
@@ -555,7 +722,9 @@ func (r *KluctlDeploymentReconciler) doFinalize(ctx context.Context, obj *kluctl
 
 	pt := pp.newTarget()
 
-	_, _ = pt.kluctlDelete(ctx, obj.Status.TargetKey.Discriminator)
+	commandResultId := uuid.NewString()
+
+	_, _ = pt.kluctlDelete(ctx, obj.Status.TargetKey.Discriminator, commandResultId)
 }
 
 // checkLegacyKluctlDeployment checks if a legacy KluctlDeployment from the old flux.kluctl.io group is present. If yes
@@ -575,7 +744,8 @@ func (r *KluctlDeploymentReconciler) checkLegacyKluctlDeployment(ctx context.Con
 		if errors2.Unwrap(err) != nil {
 			err = errors2.Unwrap(err)
 		}
-		if meta2.IsNoMatchError(err) || errors.IsNotFound(err) || discovery.IsGroupDiscoveryFailedError(err) {
+		_, isRDFErr := err.(*apiutil.ErrResourceDiscoveryFailed)
+		if apimeta.IsNoMatchError(err) || errors.IsNotFound(err) || discovery.IsGroupDiscoveryFailedError(err) || isRDFErr {
 			// legacy object not present, we're safe to continue
 			return false, nil
 		}
