@@ -3,20 +3,18 @@ package commands
 import (
 	"context"
 	"fmt"
-	"github.com/google/uuid"
-	"github.com/kluctl/kluctl/v2/pkg/deployment"
 	utils2 "github.com/kluctl/kluctl/v2/pkg/deployment/utils"
-	"github.com/kluctl/kluctl/v2/pkg/k8s"
+	"github.com/kluctl/kluctl/v2/pkg/kluctl_project"
 	k8s2 "github.com/kluctl/kluctl/v2/pkg/types/k8s"
 	"github.com/kluctl/kluctl/v2/pkg/types/result"
 	"github.com/kluctl/kluctl/v2/pkg/utils"
 	"github.com/kluctl/kluctl/v2/pkg/utils/uo"
 	"github.com/kluctl/kluctl/v2/pkg/validation"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"time"
 )
 
 type ValidateCommand struct {
-	c             *deployment.DeploymentCollection
+	targetCtx     *kluctl_project.TargetContext
 	r             *result.CommandResult
 	discriminator string
 
@@ -24,42 +22,38 @@ type ValidateCommand struct {
 	ru  *utils2.RemoteObjectUtils
 }
 
-func NewValidateCommand(ctx context.Context, discriminator string, c *deployment.DeploymentCollection, r *result.CommandResult) *ValidateCommand {
+func NewValidateCommand(discriminator string, targetCtx *kluctl_project.TargetContext, r *result.CommandResult) *ValidateCommand {
 	cmd := &ValidateCommand{
-		c:             c,
+		targetCtx:     targetCtx,
 		r:             r,
 		discriminator: discriminator,
 		dew:           utils2.NewDeploymentErrorsAndWarnings(),
 	}
-	cmd.ru = utils2.NewRemoteObjectsUtil(ctx, cmd.dew)
+	cmd.ru = utils2.NewRemoteObjectsUtil(targetCtx.SharedContext.Ctx, cmd.dew)
 	return cmd
 }
 
-func (cmd *ValidateCommand) Run(ctx context.Context, k *k8s.K8sCluster) (*result.ValidateResult, error) {
-	ret := result.ValidateResult{
-		Id:        uuid.New().String(),
-		StartTime: metav1.Now(),
-		Ready:     true,
+func (cmd *ValidateCommand) Run(ctx context.Context) *result.ValidateResult {
+	startTime := time.Now()
+	if cmd.r == nil {
+		startTime = cmd.targetCtx.KluctlProject.LoadTime
 	}
 
 	cmd.dew.Init()
 
+	ret := newValidateCommandResult(cmd.targetCtx, startTime)
+	ret.Ready = true
+
+	defer func() {
+		finishValidateResult(ret, cmd.targetCtx, cmd.dew)
+	}()
+
 	var refs []k8s2.ObjectRef
 	var renderedObjects []*uo.UnstructuredObject
 	appliedObjects := map[k8s2.ObjectRef]*uo.UnstructuredObject{}
-	var discriminator string
+	discriminator := cmd.discriminator
 
-	if cmd.c != nil && cmd.r != nil {
-		return nil, fmt.Errorf("passing both deployment collection and command result is not allowed")
-	} else if cmd.c != nil {
-		for _, d := range cmd.c.Deployments {
-			for _, o := range d.Objects {
-				ref := o.GetK8sRef()
-				refs = append(refs, ref)
-				renderedObjects = append(renderedObjects, o)
-			}
-		}
-	} else if cmd.r != nil {
+	if cmd.r != nil {
 		for _, o := range cmd.r.Objects {
 			refs = append(refs, o.Ref)
 			if o.Hook {
@@ -72,21 +66,29 @@ func (cmd *ValidateCommand) Run(ctx context.Context, k *k8s.K8sCluster) (*result
 				appliedObjects[o.Ref] = o.Applied
 			}
 		}
-		discriminator = cmd.r.TargetKey.Discriminator
+		if discriminator == "" {
+			discriminator = cmd.r.TargetKey.Discriminator
+		}
 	} else {
-		return nil, fmt.Errorf("either deployment collection or command result must be passed")
+		for _, d := range cmd.targetCtx.DeploymentCollection.Deployments {
+			for _, o := range d.Objects {
+				ref := o.GetK8sRef()
+				refs = append(refs, ref)
+				renderedObjects = append(renderedObjects, o)
+			}
+		}
+		if discriminator == "" {
+			discriminator = cmd.targetCtx.Target.Discriminator
+		}
 	}
 
-	if discriminator == "" {
-		discriminator = cmd.discriminator
-	}
-
-	err := cmd.ru.UpdateRemoteObjects(k, &cmd.discriminator, refs, true)
+	err := cmd.ru.UpdateRemoteObjects(cmd.targetCtx.SharedContext.K, &discriminator, refs, true)
 	if err != nil {
-		return nil, err
+		cmd.dew.AddError(k8s2.ObjectRef{}, err)
+		return ret
 	}
 
-	ad := utils2.NewApplyDeploymentsUtil(ctx, cmd.dew, cmd.ru, k, &utils2.ApplyUtilOptions{})
+	ad := utils2.NewApplyDeploymentsUtil(ctx, cmd.dew, cmd.ru, cmd.targetCtx.SharedContext.K, &utils2.ApplyUtilOptions{})
 	for _, o := range renderedObjects {
 		if utils.ParseBoolOrFalse(o.GetK8sAnnotation("kluctl.io/delete")) {
 			if cmd.ru.GetRemoteObject(o.GetK8sRef()) != nil {
@@ -109,7 +111,7 @@ func (cmd *ValidateCommand) Run(ctx context.Context, k *k8s.K8sCluster) (*result
 			ret.Errors = append(ret.Errors, result.DeploymentError{Ref: ref, Message: "object not found"})
 			continue
 		}
-		r := validation.ValidateObject(k, remoteObject, true, false)
+		r := validation.ValidateObject(cmd.targetCtx.SharedContext.K, remoteObject, true, false)
 		if !r.Ready {
 			ret.Ready = false
 		}
@@ -118,20 +120,7 @@ func (cmd *ValidateCommand) Run(ctx context.Context, k *k8s.K8sCluster) (*result
 		ret.Results = append(ret.Results, r.Results...)
 	}
 
-	du := utils2.NewDiffUtil(cmd.dew, cmd.ru, appliedObjects)
-	du.Swapped = true
-	du.DiffObjects(renderedObjects)
-
-	ret.Warnings = append(ret.Warnings, cmd.dew.GetWarningsList()...)
-	ret.Errors = append(ret.Errors, cmd.dew.GetErrorsList()...)
-
-	if len(du.ChangedObjects) != 0 {
-		ret.Drift = du.ChangedObjects
-	}
-
-	ret.EndTime = metav1.Now()
-
-	return &ret, nil
+	return ret
 }
 
 func (cmd *ValidateCommand) ForgetRemoteObject(ref k8s2.ObjectRef) {
