@@ -40,10 +40,10 @@ type CacheEntry struct {
 	rp         *GitRepoCache
 	url        types.GitUrl
 	mr         *git.MirroredGitRepo
-	defaultRef string
+	defaultRef types.GitRef
 	refs       map[string]string
 
-	clonedDirs   map[string]clonedDir
+	clonedDirs   map[types.GitRef]clonedDir
 	updateMutex  sync.Mutex
 	overridePath string
 }
@@ -51,7 +51,7 @@ type CacheEntry struct {
 type RepoInfo struct {
 	Url        types.GitUrl      `json:"url"`
 	RemoteRefs map[string]string `json:"remoteRefs"`
-	DefaultRef string            `json:"defaultRef"`
+	DefaultRef types.GitRef      `json:"defaultRef"`
 }
 
 type RepoOverride struct {
@@ -130,7 +130,7 @@ func (rp *GitRepoCache) GetEntry(url types.GitUrl) (*CacheEntry, error) {
 			rp:           rp,
 			url:          url,
 			mr:           nil, // mark as overridden
-			clonedDirs:   map[string]clonedDir{},
+			clonedDirs:   map[types.GitRef]clonedDir{},
 			overridePath: overridePath,
 		}
 		rp.repos[repoKey] = e
@@ -147,7 +147,7 @@ func (rp *GitRepoCache) GetEntry(url types.GitUrl) (*CacheEntry, error) {
 			rp:         rp,
 			url:        url,
 			mr:         mr,
-			clonedDirs: map[string]clonedDir{},
+			clonedDirs: map[types.GitRef]clonedDir{},
 		}
 		rp.repos[repoKey] = e
 	}
@@ -193,11 +193,17 @@ func (e *CacheEntry) Update() error {
 		return err
 	}
 
-	e.defaultRef, err = e.mr.DefaultRef()
+	defaultRefStr, err := e.mr.DefaultRef()
 	if err != nil {
 		return err
 	}
 
+	defaultRef, err := types.ParseGitRef(defaultRefStr)
+	if err != nil {
+		return err
+	}
+	e.defaultRef = defaultRef
+
 	return nil
 }
 
@@ -244,6 +250,7 @@ func (e *CacheEntry) findRef(ref string) (string, string, error) {
 		}
 		return ref, c, nil
 	default:
+		// TODO remove this compatibility code
 		ref2 := "refs/heads/" + ref
 		c, ok := e.refs[ref2]
 		if ok {
@@ -258,7 +265,7 @@ func (e *CacheEntry) findRef(ref string) (string, string, error) {
 	}
 }
 
-func (e *CacheEntry) GetClonedDir(ref string) (string, git.CheckoutInfo, error) {
+func (e *CacheEntry) GetClonedDir(ref *types.GitRef) (string, git.CheckoutInfo, error) {
 	e.updateMutex.Lock()
 	defer e.updateMutex.Unlock()
 
@@ -270,10 +277,10 @@ func (e *CacheEntry) GetClonedDir(ref string) (string, git.CheckoutInfo, error)
 
 	url := e.url
 	repoName := path.Base(url.Normalize().Path) + "-"
-	if ref == "" {
+	if ref == nil {
 		repoName += "HEAD-"
 	} else {
-		repoName += ref + "-"
+		repoName += ref.String() + "-"
 	}
 	repoName = strings.ReplaceAll(repoName, "/", "-")
 
@@ -300,13 +307,27 @@ func (e *CacheEntry) GetClonedDir(ref string) (string, git.CheckoutInfo, error)
 	}
 	defer e.mr.Unlock()
 
-	if ref == "" {
-		ref = e.defaultRef
+	if ref == nil {
+		ref = &e.defaultRef
 	}
 
-	ref2, commit, err := e.findCommit(ref)
-	if err != nil {
-		return "", git.CheckoutInfo{}, err
+	var commit string
+	var checkoutInfo git.CheckoutInfo
+	if ref.Commit != "" {
+		commit = ref.Commit
+		checkoutInfo.CheckedOutRef = *ref
+		checkoutInfo.CheckedOutCommit = ref.Commit
+	} else {
+		var ref2 string
+		ref2, commit, err = e.findCommit(ref.String())
+		if err != nil {
+			return "", git.CheckoutInfo{}, err
+		}
+		checkoutInfo.CheckedOutRef, err = types.ParseGitRef(ref2)
+		if err != nil {
+			return "", git.CheckoutInfo{}, err
+		}
+		checkoutInfo.CheckedOutCommit = commit
 	}
 
 	err = e.mr.CloneProjectByCommit(commit, p)
@@ -314,16 +335,9 @@ func (e *CacheEntry) GetClonedDir(ref string) (string, git.CheckoutInfo, error)
 		return "", git.CheckoutInfo{}, err
 	}
 
-	repoInfo, err := git.GetCheckoutInfo(p)
-	if err != nil {
-		return "", git.CheckoutInfo{}, err
-	}
-
-	repoInfo.CheckedOutRef = ref2
-
-	e.clonedDirs[ref] = clonedDir{
+	e.clonedDirs[*ref] = clonedDir{
 		dir:  p,
-		info: repoInfo,
+		info: checkoutInfo,
 	}
-	return p, repoInfo, nil
+	return p, checkoutInfo, nil
 }
