@@ -29,6 +29,7 @@ type DeploymentError struct {
 type KluctlDeploymentInfo struct {
 	Name      string `json:"name"`
 	Namespace string `json:"namespace"`
+	ClusterId string `json:"clusterId"`
 }
 
 type CommandInitiator string
@@ -76,7 +77,6 @@ type CommandInfo struct {
 	Initiator             CommandInitiator       `json:"initiator" validate:"oneof=CommandLine KluctlDeployment"`
 	StartTime             metav1.Time            `json:"startTime"`
 	EndTime               metav1.Time            `json:"endTime"`
-	KluctlDeployment      *KluctlDeploymentInfo  `json:"kluctlDeployment,omitempty"`
 	Command               string                 `json:"command,omitempty"`
 	Target                string                 `json:"target,omitempty"`
 	TargetNameOverride    string                 `json:"targetNameOverride,omitempty"`
@@ -97,7 +97,7 @@ type CommandInfo struct {
 
 type GitInfo struct {
 	Url    *types.GitUrl `json:"url"`
-	Ref    string        `json:"ref"`
+	Ref    *types.GitRef `json:"ref"`
 	SubDir string        `json:"subDir"`
 	Commit string        `json:"commit"`
 	Dirty  bool          `json:"dirty"`
@@ -126,16 +126,19 @@ type ResultObject struct {
 }
 
 type CommandResult struct {
-	Id          string                         `json:"id"`
-	ProjectKey  ProjectKey                     `json:"projectKey"`
-	TargetKey   TargetKey                      `json:"targetKey"`
-	Target      types.Target                   `json:"target"`
-	Command     CommandInfo                    `json:"command,omitempty"`
-	GitInfo     GitInfo                        `json:"gitInfo,omitempty"`
-	ClusterInfo ClusterInfo                    `json:"clusterInfo"`
-	Deployment  *types.DeploymentProjectConfig `json:"deployment,omitempty"`
+	Id               string                         `json:"id"`
+	ReconcileId      string                         `json:"reconcileId"`
+	ProjectKey       ProjectKey                     `json:"projectKey"`
+	TargetKey        TargetKey                      `json:"targetKey"`
+	Target           types.Target                   `json:"target"`
+	Command          CommandInfo                    `json:"command,omitempty"`
+	KluctlDeployment *KluctlDeploymentInfo          `json:"kluctlDeployment,omitempty"`
+	GitInfo          GitInfo                        `json:"gitInfo,omitempty"`
+	ClusterInfo      ClusterInfo                    `json:"clusterInfo"`
+	Deployment       *types.DeploymentProjectConfig `json:"deployment,omitempty"`
 
-	Objects []ResultObject `json:"objects,omitempty"`
+	RenderedObjectsHash string         `json:"renderedObjectsHash,omitempty"`
+	Objects             []ResultObject `json:"objects,omitempty"`
 
 	Errors     []DeploymentError  `json:"errors,omitempty"`
 	Warnings   []DeploymentError  `json:"warnings,omitempty"`
@@ -156,9 +159,9 @@ func (cr *CommandResult) ToReducedObjects() *CommandResult {
 	ret.Objects = make([]ResultObject, len(ret.Objects))
 	for i, o := range cr.Objects {
 		ret.Objects[i] = o
-		ret.Objects[i].Rendered = buildReducedObject(o.Rendered)
-		ret.Objects[i].Remote = buildReducedObject(o.Remote)
-		ret.Objects[i].Applied = buildReducedObject(o.Applied)
+		ret.Objects[i].Rendered = BuildReducedObject(o.Rendered)
+		ret.Objects[i].Remote = BuildReducedObject(o.Remote)
+		ret.Objects[i].Applied = BuildReducedObject(o.Applied)
 	}
 	return &ret
 }
@@ -176,21 +179,3 @@ func (ccr *CompactedCommandResult) ToNonCompacted() *CommandResult {
 	}
 	return &ret
 }
-
-type ValidateResultEntry struct {
-	Ref        k8s.ObjectRef `json:"ref"`
-	Annotation string        `json:"annotation"`
-	Message    string        `json:"message"`
-}
-
-type ValidateResult struct {
-	Id        string                `json:"id"`
-	StartTime metav1.Time           `json:"startTime"`
-	EndTime   metav1.Time           `json:"endTime"`
-	Ready     bool                  `json:"ready"`
-	Warnings  []DeploymentError     `json:"warnings,omitempty"`
-	Errors    []DeploymentError     `json:"errors,omitempty"`
-	Results   []ValidateResultEntry `json:"results,omitempty"`
-
-	Drift []ChangedObject `json:"drift,omitempty"`
-}
