@@ -6,11 +6,13 @@ import (
 	errors2 "errors"
 	"fmt"
 	types2 "github.com/aws/aws-sdk-go-v2/service/secretsmanager/types"
+	"github.com/getsops/sops/v3/cmd/sops/formats"
 	"github.com/kluctl/go-jinja2"
 	"github.com/kluctl/kluctl/v2/pkg/k8s"
 	"github.com/kluctl/kluctl/v2/pkg/repocache"
 	"github.com/kluctl/kluctl/v2/pkg/sops"
 	"github.com/kluctl/kluctl/v2/pkg/sops/decryptor"
+	"github.com/kluctl/kluctl/v2/pkg/status"
 	"github.com/kluctl/kluctl/v2/pkg/types"
 	k8s2 "github.com/kluctl/kluctl/v2/pkg/types/k8s"
 	"github.com/kluctl/kluctl/v2/pkg/utils"
@@ -18,7 +20,6 @@ import (
 	"github.com/kluctl/kluctl/v2/pkg/vars/aws"
 	"github.com/kluctl/kluctl/v2/pkg/vars/vault"
 	"github.com/kluctl/kluctl/v2/pkg/yaml"
-	"go.mozilla.org/sops/v3/cmd/sops/formats"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"os"
@@ -51,9 +52,9 @@ func NewVarsLoader(ctx context.Context, k *k8s.K8sCluster, sops *decryptor.Decry
 	}
 }
 
-func (v *VarsLoader) LoadVarsList(varsCtx *VarsCtx, varsList []*types.VarsSource, searchDirs []string, rootKey string) error {
+func (v *VarsLoader) LoadVarsList(ctx context.Context, varsCtx *VarsCtx, varsList []*types.VarsSource, searchDirs []string, rootKey string) error {
 	for _, source := range varsList {
-		err := v.LoadVars(varsCtx, source, searchDirs, rootKey)
+		err := v.LoadVars(ctx, varsCtx, source, searchDirs, rootKey)
 		if err != nil {
 			return err
 		}
@@ -61,7 +62,7 @@ func (v *VarsLoader) LoadVarsList(varsCtx *VarsCtx, varsList []*types.VarsSource
 	return nil
 }
 
-func (v *VarsLoader) LoadVars(varsCtx *VarsCtx, sourceIn *types.VarsSource, searchDirs []string, rootKey string) error {
+func (v *VarsLoader) LoadVars(ctx context.Context, varsCtx *VarsCtx, sourceIn *types.VarsSource, searchDirs []string, rootKey string) error {
 	if sourceIn.RenderedVars != nil && len(sourceIn.RenderedVars.Object) != 0 {
 		return fmt.Errorf("renderedVars is not allowed here")
 	}
@@ -96,6 +97,7 @@ func (v *VarsLoader) LoadVars(varsCtx *VarsCtx, sourceIn *types.VarsSource, sear
 	}
 
 	var newVars *uo.UnstructuredObject
+	var sensitive bool
 	if source.Values != nil {
 		newVars = source.Values
 		if rootKey != "" {
@@ -104,21 +106,25 @@ func (v *VarsLoader) LoadVars(varsCtx *VarsCtx, sourceIn *types.VarsSource, sear
 			})
 		}
 	} else if source.File != nil {
-		newVars, err = v.loadFile(varsCtx, *source.File, ignoreMissing, searchDirs)
+		newVars, sensitive, err = v.loadFile(varsCtx, *source.File, ignoreMissing, searchDirs)
 	} else if source.Git != nil {
-		newVars, err = v.loadGit(varsCtx, source.Git, ignoreMissing)
+		newVars, sensitive, err = v.loadGit(ctx, varsCtx, source.Git, ignoreMissing)
 	} else if source.ClusterConfigMap != nil {
 		newVars, err = v.loadFromK8sObject(varsCtx, *source.ClusterConfigMap, "ConfigMap", ignoreMissing, false)
 	} else if source.ClusterSecret != nil {
 		newVars, err = v.loadFromK8sObject(varsCtx, *source.ClusterSecret, "Secret", ignoreMissing, true)
+		sensitive = true
 	} else if source.SystemEnvVars != nil {
 		newVars, err = v.loadSystemEnvs(varsCtx, &source, ignoreMissing, rootKey)
+		sensitive = true
 	} else if source.Http != nil {
-		newVars, err = v.loadHttp(varsCtx, &source, ignoreMissing)
+		newVars, sensitive, err = v.loadHttp(varsCtx, &source, ignoreMissing)
 	} else if source.AwsSecretsManager != nil {
 		newVars, err = v.loadAwsSecretsManager(varsCtx, &source, ignoreMissing)
+		sensitive = true
 	} else if source.Vault != nil {
 		newVars, err = v.loadVault(varsCtx, &source, ignoreMissing)
+		sensitive = true
 	} else {
 		return fmt.Errorf("invalid vars source")
 	}
@@ -126,6 +132,12 @@ func (v *VarsLoader) LoadVars(varsCtx *VarsCtx, sourceIn *types.VarsSource, sear
 		return err
 	}
 
+	if sourceIn.Sensitive != nil {
+		// override the default
+		sensitive = *sourceIn.Sensitive
+	}
+
+	sourceIn.RenderedSensitive = sensitive
 	sourceIn.RenderedVars = newVars.Clone()
 
 	if source.NoOverride == nil || !*source.NoOverride {
@@ -146,32 +158,33 @@ func (v *VarsLoader) mergeVars(varsCtx *VarsCtx, newVars *uo.UnstructuredObject,
 	}
 }
 
-func (v *VarsLoader) loadFile(varsCtx *VarsCtx, path string, ignoreMissing bool, searchDirs []string) (*uo.UnstructuredObject, error) {
+func (v *VarsLoader) loadFile(varsCtx *VarsCtx, path string, ignoreMissing bool, searchDirs []string) (*uo.UnstructuredObject, bool, error) {
 	rendered, err := varsCtx.RenderFile(path, searchDirs)
 	if err != nil {
 		// TODO the Jinja2 renderer should be able to better report this error
-		if ignoreMissing && err.Error() == fmt.Sprintf("template %s not found", path) {
-			return uo.New(), nil
+		notFound := err.Error() == fmt.Sprintf("template %s not found", path) || err.Error() == fmt.Sprintf("absolute path of %s could not be resolved", path)
+		if ignoreMissing && notFound {
+			return uo.New(), false, nil
 		}
-		return nil, fmt.Errorf("failed to render vars file %s: %w", path, err)
+		return nil, false, fmt.Errorf("failed to render vars file %s: %w", path, err)
 	}
 
 	format := formats.FormatForPath(path)
-	decrypted, _, err := sops.MaybeDecrypt(v.sops, []byte(rendered), format, format)
+	decrypted, sensitive, err := sops.MaybeDecrypt(v.sops, []byte(rendered), format, format)
 	if err != nil {
-		return nil, fmt.Errorf("failed to decrypt vars file %s: %w", path, err)
+		return nil, false, fmt.Errorf("failed to decrypt vars file %s: %w", path, err)
 	}
 	rendered = string(decrypted)
 
 	newVars := uo.New()
 	err = yaml.ReadYamlString(rendered, newVars)
 	if err != nil {
-		return nil, err
+		return nil, false, err
 	}
 	if err != nil {
-		return nil, fmt.Errorf("failed to load vars from %s: %w", path, err)
+		return nil, false, fmt.Errorf("failed to load vars from %s: %w", path, err)
 	}
-	return newVars, nil
+	return newVars, sensitive, nil
 }
 
 func (v *VarsLoader) loadSystemEnvs(varsCtx *VarsCtx, source *types.VarsSource, ignoreMissing bool, rootKey string) (*uo.UnstructuredObject, error) {
@@ -260,15 +273,21 @@ func (v *VarsLoader) loadVault(varsCtx *VarsCtx, source *types.VarsSource, ignor
 	return v.loadFromString(varsCtx, *secret)
 }
 
-func (v *VarsLoader) loadGit(varsCtx *VarsCtx, gitFile *types.VarsSourceGit, ignoreMissing bool) (*uo.UnstructuredObject, error) {
+func (v *VarsLoader) loadGit(ctx context.Context, varsCtx *VarsCtx, gitFile *types.VarsSourceGit, ignoreMissing bool) (*uo.UnstructuredObject, bool, error) {
 	ge, err := v.rp.GetEntry(gitFile.Url)
 	if err != nil {
-		return nil, err
+		return nil, false, err
+	}
+
+	if gitFile.Ref != nil && gitFile.Ref.Ref != "" {
+		status.Deprecation(ctx, "git-vars-string-ref", "Passing 'ref' as string into git vars source is "+
+			"deprecated and support for this will be removed in a future version of Kluctl. Please refer to the "+
+			"documentation for details: https://kluctl.io/docs/kluctl/reference/templating/variable-sources/#git")
 	}
 
 	clonedDir, _, err := ge.GetClonedDir(gitFile.Ref)
 	if err != nil {
-		return nil, fmt.Errorf("failed to load vars from git repository %s: %w", gitFile.Url.String(), err)
+		return nil, false, fmt.Errorf("failed to load vars from git repository %s: %w", gitFile.Url.String(), err)
 	}
 
 	return v.loadFile(varsCtx, gitFile.Path, ignoreMissing, []string{clonedDir})
